import 'package:dio/dio.dart';
import '../dio_extend.dart';
import 'http_config.dart';

/// dio扩展
class EDio {
  Dio _dio;
  static var config = new Config();
  static EDio _instance = EDio._internal();

  factory EDio() => _instance;
  SignStrategy _signStrategy;

  ///通用全局单例，第一次使用时初始化
  EDio._internal() {
    if (null == _dio) {
      _dio = new Dio(new BaseOptions(
          baseUrl: config.baseUrl, connectTimeout: config.connectTimeout, receiveTimeout: config.receiveTimeout));
    }
  }

  /// GET请求方法
  /// @param {string} uri  请求地址
  /// @param {map} params 请求入参

  Future<Response> get<T>(String uri, {Map<String, dynamic> params, Options options, String cookie}) =>
      _request('GET', uri, queryParameters: params, options: options, cookie: cookie);

  /// POST请求方法
  /// @param {string} uri  请求地址
  /// @param {map} params 请求入参
  Future<Response> post<T>(String uri, {Map<String, dynamic> params, Options options, String cookie}) =>
      _request('POST', uri, queryParameters: params, options: options, cookie: cookie);

  Future<Response> request<T>(String uri, String method,
          {Map<String, dynamic> params, Options options, String cookie}) =>
      _request(method, uri, queryParameters: params, options: options, cookie: cookie);

  /// 通用请求方法
  /// @param {string} method  请求类型
  /// @param {string} uri  请求地址
  /// @param {map} queryParameters 请求入参
  /// @param {Options} options 请求配置
  /// @param {string} cookie 登录态信息
  Future<Response> _request(String method, String uri,
      {Map<String, dynamic> queryParameters, Options options, String cookie}) async {
    Options op = Options();
    if (options != null) {
      op = options;
    } else {
      op = Options();
      op.method = method;
      if (method.toLowerCase() == 'post') {
        op.contentType = Headers.formUrlEncodedContentType;
      }

      if (cookie != null) {
        op.headers['Cookie'] = cookie;
      }
    }

    Map<String, dynamic> parameters = new Map();
    if (_signStrategy != null) {
      parameters.addAll(_signStrategy.sign(queryParameters));
    } else {
      parameters.addAll(queryParameters);
    }

    return await _dio.request(uri, queryParameters: parameters, options: op);
  }

  /// 设置接口签名策略
  void setStrategy(SignStrategy signStrategy) {
    _signStrategy = signStrategy;
  }
}
